import collections
import StringIO
import sys
import urlparse
import xmlrpclib

from docutils import io, readers
from docutils.core import publish_doctree, Publisher
from docutils.transforms import TransformError
from progress.bar import Bar

import recliner

##############################################################################
##### Copied from PyPI #######################################################
##############################################################################


def pypi_render(source):
    """
    Copied (and slightly adapted) from pypi.description_tools
    """
    ALLOWED_SCHEMES = '''file ftp gopher hdl http https imap mailto mms news
        nntp prospero rsync rtsp rtspu sftp shttp sip sips snews svn svn+ssh
        telnet wais irc'''.split()

    settings_overrides = {
        "raw_enabled": 0,  # no raw HTML code
        "file_insertion_enabled": 0,  # no file/URL access
        "halt_level": 2,  # at warnings or errors, raise an exception
        "report_level": 5,  # never report problems with the reST code
    }

    # capture publishing errors, they go to stderr
    old_stderr = sys.stderr
    sys.stderr = s = StringIO.StringIO()
    parts = None

    try:
        # Convert reStructuredText to HTML using Docutils.
        document = publish_doctree(source=source,
            settings_overrides=settings_overrides)

        for node in document.traverse():
            if node.tagname == '#text':
                continue
            if node.hasattr('refuri'):
                uri = node['refuri']
            elif node.hasattr('uri'):
                uri = node['uri']
            else:
                continue
            o = urlparse.urlparse(uri)
            if o.scheme not in ALLOWED_SCHEMES:
                raise TransformError('link scheme not allowed')

        # now turn the transformed document into HTML
        reader = readers.doctree.Reader(parser_name='null')
        pub = Publisher(reader, source=io.DocTreeInput(document),
            destination_class=io.StringOutput)
        pub.set_writer('html')
        pub.process_programmatic_settings(None, settings_overrides, None)
        pub.set_destination(None, None)
        pub.publish()
        parts = pub.writer.parts

    except:
        pass

    sys.stderr = old_stderr

    # original text if publishing errors occur
    if parts is None or len(s.getvalue()) > 0:
        return None
    else:
        return parts['body']


##############################################################################
##############################################################################
##############################################################################

client = xmlrpclib.Server("http://pypi.python.org/pypi")

counter = collections.Counter()

unexpected = []

packages = client.list_packages()

for package in Bar("Processing", max=len(packages)).iter(packages):
    for version in client.package_releases(package, True):
        release_data = client.release_data(package, version)

        description = release_data.get("description", "")

        if description is None:
            continue

        html = recliner.htmlize(description)
        pypi = pypi_render(description)

        counter["total"] += 1

        if html.rendered:
            counter["recliner"] += 1
        if pypi is not None:
            counter["pypi"] += 1

        if not html.rendered and pypi is not None:
            unexpected.append((package, version))


for key, count in counter.items():
    print key, "=>", count

if unexpected:
    print ""
    print "Recliner => No; PyPI => Yes"
    print "==========================="

    for package, version in unexpected:
        print "    ", package.encode("utf-8"), "=>", version.encode("utf8")



from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from . import __about__
from .renderer import render, clean, htmlize


__all__ = ["render", "clean", "htmlize"] + __about__.__all__


# - Meta Information -
# This is pretty ugly
for attr in __about__.__all__:
    if hasattr(__about__, attr):
        globals()[attr] = getattr(__about__, attr)
# - End Meta Information -



import argparse
import codecs
import locale
import sys

from . import render


locale.setlocale(locale.LC_ALL, "")


def main():
    parser = argparse.ArgumentParser(description="renders reStructuredText")
    parser.add_argument("source",
        nargs="?",
        help="file to open and render",
    )

    args = parser.parse_args()

    if args.source:
        with codecs.open(args.source, "r", "utf-8") as fp:
            text = fp.read()
    else:
        text = sys.stdin.read()

    rendered = render(text)

    sys.stdout.write(rendered.encode("utf-8"))
    sys.stdout.flush()


if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import io

from docutils.core import publish_parts
from docutils.writers.html4css1 import HTMLTranslator, Writer
from docutils.utils import SystemMessage

import bleach

from . import six


__all__ = ["render", "clean", "htmlize"]


ALLOWED_TAGS = [
    # Bleach Defaults
    "a", "abbr", "acronym", "b", "blockquote", "code", "em", "i", "li", "ol",
    "strong", "ul",

    # Recliner Additions
    "br", "cite", "col", "colgroup", "dd", "div", "dl", "dt", "h1", "h2", "h3",
    "h4", "h5", "h6", "img", "p", "pre", "span", "table", "tbody", "td", "th",
    "thead", "tr", "tt",
]

ALLOWED_ATTRIBUTES = {
    # Bleach Defaults
    "a": ["href", "title"],
    "abbr": ["title"],
    "acronym": ["title"],
}

ALLOWED_ATTRIBUTES.update({
    # Recliner Additions
    "img": ["src"],
    "span": ["class"],
})

ALLOWED_STYLES = []


class HTML(six.text_type):

    raw = None
    rendered = False


def render(raw):
    settings = {
        # Cloaking email addresses provides a small amount of additional
        #   privacy protection for email addresses inside of a chunk of ReST.
        "cloak_email_addresses": True,

        # Prevent a lone top level heading from being promoted to document
        #   title, and thus second level headings from being promoted to top
        #   level.
        "doctitle_xform": True,

        # Set our initial header level
        "initial_header_level": 2,

        # Prevent local files from being included into the rendered output.
        #   This is a security concern because people can insert files
        #   that are part of the system, such as /etc/passwd.
        "file_insertion_enabled": False,

        # Halt rendering and throw an exception if there was any errors or
        #   warnings from docutils.
        "halt_level": 2,

        # Output math blocks as LaTeX that can be interpreted by MathJax for
        #   a prettier display of Math formulas.
        "math_output": "MathJax",

        # Disable raw html as enabling it is a security risk, we do not want
        #   people to be able to include any old HTML in the final output.
        "raw_enabled": False,

        # Disable all system messages from being reported.
        "report_level": 5,

        # Use typographic quotes, and transform --, ---, and ... into their
        #   typographic counterparts.
        "smart_quotes": True,

        # Strip all comments from the rendered output.
        "strip_comments": True,

        # Use the short form of syntax highlighting so that the generated
        #   Pygments CSS can be used to style the output.
        "syntax_highlight": "short",

        # Use a io.StringIO as the warning stream to prevent warnings from
        #   being printed to sys.stderr.
        "warning_stream": io.StringIO(),
    }

    writer = Writer()
    writer.translator_class = HTMLTranslator

    try:
        parts = publish_parts(raw,
                    writer=writer,
                    settings_overrides=settings,
                )
    except SystemMessage as exc:
        raise ValueError(exc.message)

    rendered = parts.get("fragment")

    if rendered is None:
        raise ValueError("There was no rendered value")

    return rendered


def clean(html):
    def nofollow(attrs, new=False):
        if attrs["href"].startswith("mailto:"):
            return attrs
        attrs["rel"] = "nofollow"
        return attrs

    # Clean the output using Bleach
    cleaned = bleach.clean(html,
                tags=ALLOWED_TAGS,
                attributes=ALLOWED_ATTRIBUTES,
                styles=ALLOWED_STYLES,
            )

    # Bleach Linkify makes it easy to modify links, however, we will not be
    #   using it to create additional links.
    cleaned = bleach.linkify(cleaned,
                callbacks=[
                    lambda attrs, new: attrs if not new else None,
                    nofollow,
                ],
                skip_pre=True,
                parse_email=False,
            )

    return cleaned


def htmlize(text):
    # Try to render the text, or use the text itself
    try:
        html = render(text)
        rendered = True
    except ValueError:
        html = text
        rendered = False

    # Clean the (possibly) rendered text
    cleaned = clean(html)

    cleaned = HTML(cleaned)
    cleaned.raw = text
    cleaned.rendered = rendered

    return cleaned



#!/usr/bin/env python
from setuptools import setup, find_packages

__about__ = {}

with open("recliner/__about__.py") as fp:
    exec(fp.read(), None, __about__)

setup(
    name=__about__["__title__"],
    version=__about__["__version__"],

    description=__about__["__summary__"],
    long_description=open("README.rst").read(),
    url=__about__["__uri__"],
    license=__about__["__license__"],

    author=__about__["__author__"],
    author_email=__about__["__email__"],

    install_requires=[
        "bleach>=1.2",
        "docutils>=0.9",
        "Pygments",
    ],
    extras_require={
        "tests": [
            "pep8",
            "pylint",
            "pytest",
            "pytest-cov",
        ],
    },
    tests_require=[
        "pep8",
        "pytest",
        "pytest-cov",
    ],

    packages=find_packages(exclude=["tests"]),

    entry_points={
        "console_scripts": [
            "recliner = recliner.__main__:main",
        ],
    },

    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],

    zip_safe=False,
)



from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "recliner"
__summary__ = ""
__uri__ = "https://github.com/crateio/recliner/"

__version__ = "0.2.1"

__author__ = "Donald Stufft"
__email__ = "donald.stufft@gmail.com"

__license__ = "Simplified BSD"
__copyright__ = "Copyright 2012 Donald Stufft"
